package dev.kske.chess.board;

import java.util.Objects;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Move.java</strong><br>
 * Created: <strong>02.07.2019</strong><br>
 * 
 * @since Chess v0.1-alpha
 * @author Kai S. K. Engelbart
 */
public class Move {

	protected final Position	pos, dest;
	protected final int			xDist, yDist, xSign, ySign;
	public Type				type;

	public Move(Position pos, Position dest, Type type) {
		this.pos	= pos;
		this.dest	= dest;
		this.type	= type;
		xDist		= Math.abs(dest.x - pos.x);
		yDist		= Math.abs(dest.y - pos.y);
		xSign		= (int) Math.signum(dest.x - pos.x);
		ySign		= (int) Math.signum(dest.y - pos.y);
	}

	public Move(Position pos, Position dest) { this(pos, dest, Type.NORMAL); }

	public Move(int xPos, int yPos, int xDest, int yDest) { this(new Position(xPos, yPos), new Position(xDest, yDest)); }

	public static Move fromLAN(String move) { return new Move(Position.fromLAN(move.substring(0, 2)), Position.fromLAN(move.substring(2))); }

	public String toLAN() { return getPos().toLAN() + getDest().toLAN(); }

	public boolean isHorizontal() { return getyDist() == 0; }

	public boolean isVertical() { return getxDist() == 0; }

	public boolean isDiagonal() { return getxDist() == getyDist(); }

	@Override
	public String toString() { return String.format("%s -> %s", getPos(), getDest()); }

	@Override
	public int hashCode() { return Objects.hash(getDest(), getPos(), getType(), getxDist(), getxSign(), getyDist(), getySign()); }

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Move other = (Move) obj;
		return Objects.equals(getDest(), other.getDest()) && Objects.equals(getPos(), other.getPos()) && getType() == other.getType()
				&& getxDist() == other.getxDist() && getxSign() == other.getxSign() && getyDist() == other.getyDist()
				&& getySign() == other.getySign();
	}

	public Position getPos() { return pos; }

	public Position getDest() { return dest; }

	public int getxDist() { return xDist; }

	public int getyDist() { return yDist; }

	public int getxSign() { return xSign; }

	public int getySign() { return ySign; }

	public Type getType() { return type; }

	public static enum Type {
		NORMAL, PAWN_PROMOTION, CASTLING, EN_PASSANT, UNKNOWN
	}
}
